@@ -51,33 +51,14 @@
 static const char *version_str = "2.00 29/2/2000 Fons Rademakers";
 
 
-#include <linux/version.h>
 #include <linux/module.h>
 #include <linux/slab.h>
-#include <linux/proc_fs.h>
-#include <linux/ioport.h>
-#include <linux/sysctl.h>
-#include <asm/errno.h>
-#include <asm/io.h>
-#include <linux/types.h>
 #include <linux/i2c.h>
-#include "version.h"
-#include "sensors.h"
+#include <linux/i2c-proc.h>
 #include <linux/init.h>
+#include "version.h"
 
-#ifdef MODULE_LICENSE
 MODULE_LICENSE("GPL");
-#endif
-
-#if (LINUX_VERSION_CODE < KERNEL_VERSION(2,2,18)) || \
-    (LINUX_VERSION_CODE == KERNEL_VERSION(2,3,0))
-#define init_MUTEX(s) do { *(s) = MUTEX; } while(0)
-#endif
-
-#ifndef THIS_MODULE
-#define THIS_MODULE NULL
-#endif
-
 
 #undef AUTODETECT		/* try to autodetect MaxiLife version */
 /*#define AUTODETECT*/
@@ -133,7 +114,7 @@ SENSORS_INSMOD_1(maxilife);
 #define FAN_FROM_REG(val)      ((val)==0xfe ? 0 : (val)==0xff ? -1 : \
                                 (val)==0x00 ? -1 : (1171860 / (val)))
 
-extern inline u8 FAN_TO_REG(long rpm)
+static inline u8 FAN_TO_REG(long rpm)
 {
 	if (rpm == 0)
 		return 255;
@@ -241,11 +222,6 @@ extern inline u8 FAN_TO_REG(long rpm)
                                 (nr)==5 ? ((val) * 823 - 149140) : 0)
 
 
-#ifdef MODULE
-extern int init_module(void);
-extern int cleanup_module(void);
-#endif				/* MODULE */
-
 /* The following product codenames apply:
      Cristal/Geronimo: HP KAYAK XU/XAs
                        (Dual Pentium II Slot 1, Deschutes/Klamath)
@@ -262,6 +238,7 @@ enum sensor_type { fan, temp, vid, pll, lcd, alarm };
    client is allocated. We assume MaxiLife will only be present on the
    SMBus and not on the ISA bus. */
 struct maxi_data {
+	struct i2c_client client;
 	struct semaphore lock;
 	int sysctl_id;
 	enum maxi_type type;
@@ -288,22 +265,10 @@ struct maxi_data {
 };
 
 
-#ifdef MODULE
-static
-#else
-extern
-#endif
-int __init sensors_maxi_init(void);
-static int __init maxi_cleanup(void);
-
 static int maxi_attach_adapter(struct i2c_adapter *adapter);
 static int maxi_detect(struct i2c_adapter *adapter, int address,
 		       unsigned short flags, int kind);
 static int maxi_detach_client(struct i2c_client *client);
-static int maxi_command(struct i2c_client *client, unsigned int cmd,
-			void *arg);
-static void maxi_inc_use(struct i2c_client *client);
-static void maxi_dec_use(struct i2c_client *client);
 
 static int maxi_read_value(struct i2c_client *client, u8 register);
 static int maxi_read_token(struct i2c_client *client, u16 token);
@@ -341,19 +306,14 @@ static void maxi_alarms(struct i2c_client *client, int operation,
 /* The driver. I choose to use type i2c_driver, as at is identical to
    the smbus_driver. */
 static struct i2c_driver maxi_driver = {
-	/* name */ "HP MaxiLife driver",
-	/* id */ I2C_DRIVERID_MAXILIFE,
-	/* flags */ I2C_DF_NOTIFY,
-	/* attach_adapter */ &maxi_attach_adapter,
-	/* detach_client */ &maxi_detach_client,
-	/* command */ &maxi_command,
-	/* inc_use */ &maxi_inc_use,
-	/* dec_use */ &maxi_dec_use
+	.owner		= THIS_MODULE,
+	.name		= "HP MaxiLife driver",
+	.id		= I2C_DRIVERID_MAXILIFE,
+	.flags		= I2C_DF_NOTIFY,
+	.attach_adapter	= maxi_attach_adapter,
+	.detach_client	= maxi_detach_client,
 };
 
-/* Used by maxi_init/cleanup */
-static int __initdata maxi_initialized = 0;
-
 static int maxi_id = 0;
 
 /* Default firmware version. Use module option "maxi_version"
@@ -361,6 +321,49 @@ static int maxi_id = 0;
 static int maxi_version = cristal;
 
 /* The /proc/sys entries */
+
+/* -- SENSORS SYSCTL START -- */
+#define MAXI_SYSCTL_FAN1   1101	/* Rotations/min */
+#define MAXI_SYSCTL_FAN2   1102	/* Rotations/min */
+#define MAXI_SYSCTL_FAN3   1103	/* Rotations/min */
+#define MAXI_SYSCTL_FAN4   1104	/* Rotations/min */
+#define MAXI_SYSCTL_TEMP1  1201	/* Degrees Celcius */
+#define MAXI_SYSCTL_TEMP2  1202	/* Degrees Celcius */
+#define MAXI_SYSCTL_TEMP3  1203	/* Degrees Celcius */
+#define MAXI_SYSCTL_TEMP4  1204	/* Degrees Celcius */
+#define MAXI_SYSCTL_TEMP5  1205	/* Degrees Celcius */
+#define MAXI_SYSCTL_TEMP6  1206	/* Degrees Celcius */
+#define MAXI_SYSCTL_PLL    1301	/* MHz */
+#define MAXI_SYSCTL_VID1   1401	/* Volts / 6.337, for nba just Volts */
+#define MAXI_SYSCTL_VID2   1402	/* Volts */
+#define MAXI_SYSCTL_VID3   1403	/* Volts */
+#define MAXI_SYSCTL_VID4   1404	/* Volts */
+#define MAXI_SYSCTL_VID5   1405	/* Volts */
+#define MAXI_SYSCTL_LCD1   1501	/* Line 1 of LCD */
+#define MAXI_SYSCTL_LCD2   1502	/* Line 2 of LCD */
+#define MAXI_SYSCTL_LCD3   1503	/* Line 3 of LCD */
+#define MAXI_SYSCTL_LCD4   1504	/* Line 4 of LCD */
+#define MAXI_SYSCTL_ALARMS 2001	/* Bitvector (see below) */
+
+#define MAXI_ALARM_VID4      0x0001
+#define MAXI_ALARM_TEMP2     0x0002
+#define MAXI_ALARM_VID1      0x0004
+#define MAXI_ALARM_VID2      0x0008
+#define MAXI_ALARM_VID3      0x0010
+#define MAXI_ALARM_PLL       0x0080
+#define MAXI_ALARM_TEMP4     0x0100
+#define MAXI_ALARM_TEMP5     0x0200
+#define MAXI_ALARM_FAN1      0x1000
+#define MAXI_ALARM_FAN2      0x2000
+#define MAXI_ALARM_FAN3      0x4000
+
+#define MAXI_ALARM_FAN       0x0100	/* To be used with  MaxiLife'99 */
+#define MAXI_ALARM_VID       0x0200	/* The MSB specifies which sensor */
+#define MAXI_ALARM_TEMP      0x0400	/* in the alarm group failed, i.e.: */
+#define MAXI_ALARM_VADD      0x0800	/* 0x0402 = TEMP2 failed = CPU2 temp */
+
+/* -- SENSORS SYSCTL END -- */
+
 /* These files are created for each detected MaxiLife processor.
    This is just a template; though at first sight, you might think we
    could use a statically allocated list, we need some way to get back
@@ -416,7 +419,7 @@ static ctl_table maxi_dir_table_template[] = {
     - maxi_driver is inserted (when this module is loaded), for each
       available adapter
     - when a new adapter is inserted (and maxi_driver is still present) */
-int maxi_attach_adapter(struct i2c_adapter *adapter)
+static int maxi_attach_adapter(struct i2c_adapter *adapter)
 {
 	return i2c_detect(adapter, &addr_data, maxi_detect);
 }
@@ -427,9 +430,9 @@ int maxi_detect(struct i2c_adapter *adapter, int address,
 {
 	struct i2c_client *new_client;
 	struct maxi_data *data;
-	enum maxi_type type;
+	enum maxi_type type = 0;
 	int i, j, err = 0;
-	const char *type_name, *client_name;
+	const char *type_name = NULL, *client_name = NULL;
 
 	if (!i2c_check_functionality(adapter, I2C_FUNC_SMBUS_BYTE_DATA))
 		goto ERROR0;
@@ -437,15 +440,13 @@ int maxi_detect(struct i2c_adapter *adapter, int address,
 	/* OK. For now, we presume we have a valid client. We now create the
 	   client structure, even though we cannot fill it completely yet.
 	   But it allows us to access maxi_{read,write}_value. */
-	if (!(new_client = kmalloc(sizeof(struct i2c_client) +
-				   sizeof(struct maxi_data),
-				   GFP_KERNEL))) {
+	if (!(data = kmalloc(sizeof(struct maxi_data), GFP_KERNEL))) {
 		err = -ENOMEM;
 		goto ERROR0;
 	}
 
 	/* Fill the new client structure with data */
-	data = (struct maxi_data *) (new_client + 1);
+	new_client = &data->client;
 	new_client->addr = address;
 	new_client->data = data;
 	new_client->adapter = adapter;
@@ -614,8 +615,7 @@ int maxi_detect(struct i2c_adapter *adapter, int address,
 
 	/* Register a new directory entry with module sensors */
 	if ((err = i2c_register_entry(new_client, type_name,
-					  maxi_dir_table_template,
-					  THIS_MODULE)) < 0)
+					  maxi_dir_table_template)) < 0)
 		goto ERROR4;
 	data->sysctl_id = err;
 
@@ -628,7 +628,7 @@ int maxi_detect(struct i2c_adapter *adapter, int address,
       ERROR4:
 	i2c_detach_client(new_client);
       ERROR2:
-	kfree(new_client);
+	kfree(data);
       ERROR0:
 	return err;
 }
@@ -637,7 +637,7 @@ int maxi_detect(struct i2c_adapter *adapter, int address,
     - maxi_driver is removed (when this module is unloaded)
     - when an adapter is removed which has a maxi client (and maxi_driver
       is still present). */
-int maxi_detach_client(struct i2c_client *client)
+static int maxi_detach_client(struct i2c_client *client)
 {
 	int err;
 
@@ -649,41 +649,18 @@ int maxi_detach_client(struct i2c_client *client)
 		    ("maxilife: Client deregistration failed, client not detached.\n");
 		return err;
 	}
-	kfree(client);
+	kfree(client->data);
 	return 0;
 }
 
-/* No commands defined yet */
-int maxi_command(struct i2c_client *client, unsigned int cmd, void *arg)
-{
-	return 0;
-}
-
-/* Nothing here yet */
-void maxi_inc_use(struct i2c_client *client)
-{
-#ifdef MODULE
-	MOD_INC_USE_COUNT;
-#endif
-}
-
-/* Nothing here yet */
-void maxi_dec_use(struct i2c_client *client)
-{
-#ifdef MODULE
-	MOD_DEC_USE_COUNT;
-#endif
-}
-
-
 /* Read byte from specified register (-1 in case of error, value otherwise). */
-int maxi_read_value(struct i2c_client *client, u8 reg)
+static int maxi_read_value(struct i2c_client *client, u8 reg)
 {
 	return i2c_smbus_read_byte_data(client, reg);
 }
 
 /* Read the byte value for a MaxiLife token (-1 in case of error, value otherwise */
-int maxi_read_token(struct i2c_client *client, u16 token)
+static int maxi_read_token(struct i2c_client *client, u16 token)
 {
 	u8 lowToken, highToken;
 	int error, value;
@@ -749,7 +726,7 @@ int maxi_read_token(struct i2c_client *client, u16 token)
 
 #ifndef NOWRITE
 /* Write byte to specified register (-1 in case of error, 0 otherwise). */
-int maxi_write_value(struct i2c_client *client, u8 reg, u8 value)
+static int maxi_write_value(struct i2c_client *client, u8 reg, u8 value)
 {
 	return i2c_smbus_write_byte_data(client, reg, value);
 }
@@ -822,7 +799,7 @@ int maxi_write_token_loop(struct i2c_client *client, u16 token, u8 len,
 }
 
 /* Called when we have found a new MaxiLife. It should set limits, etc. */
-void maxi_init_client(struct i2c_client *client)
+static void maxi_init_client(struct i2c_client *client)
 {
 	struct maxi_data *data = client->data;
 
@@ -834,7 +811,7 @@ void maxi_init_client(struct i2c_client *client)
 	}
 }
 
-void maxi_update_client(struct i2c_client *client)
+static void maxi_update_client(struct i2c_client *client)
 {
 	struct maxi_data *data = client->data;
 	int i;
@@ -1386,56 +1363,24 @@ void maxi_alarms(struct i2c_client *client, int operation, int ctl_name,
 	}
 }
 
-int __init sensors_maxi_init(void)
+static int __init sm_maxilife_init(void)
 {
-	int res;
-
 	printk("maxilife: Version %s (lm_sensors %s (%s))\n", version_str,
 	       LM_VERSION, LM_DATE);
-	maxi_initialized = 0;
-
-	if ((res = i2c_add_driver(&maxi_driver))) {
-		printk
-		    ("maxilife: Driver registration failed, module not inserted.\n");
-		maxi_cleanup();
-		return res;
-	}
-	maxi_initialized++;
-	return 0;
+	return i2c_add_driver(&maxi_driver);
 }
 
-int __init maxi_cleanup(void)
+static void __exit sm_maxilife_exit(void)
 {
-	int res;
-
-	if (maxi_initialized >= 1) {
-		if ((res = i2c_del_driver(&maxi_driver))) {
-			printk
-			    ("maxilife: Driver deregistration failed, module not removed.\n");
-			return res;
-		}
-		maxi_initialized--;
-	}
-	return 0;
+	i2c_del_driver(&maxi_driver);
 }
 
-EXPORT_NO_SYMBOLS;
 
-#ifdef MODULE
 
 MODULE_AUTHOR("Fons Rademakers <Fons.Rademakers@cern.ch>");
 MODULE_DESCRIPTION("HP MaxiLife driver");
 MODULE_PARM(maxi_version, "i");
 MODULE_PARM_DESC(maxi_version, "MaxiLife firmware version");
 
-int init_module(void)
-{
-	return sensors_maxi_init();
-}
-
-int cleanup_module(void)
-{
-	return maxi_cleanup();
-}
-
-#endif				/* MODULE */
+module_init(sm_maxilife_init);
+module_exit(sm_maxilife_exit);
